//
//  CommentInputView.swift
//  BJVideoPlayerUI
//
//  Created by ney on 2023/6/5.
//  Copyright © 2023 BaijiaYun. All rights reserved.
//

import UIKit
import BJSwiftBase

public class CommentInputView: UIView {
    // MARK: - api
    public var sendActionCallback: ((String) -> Void)?
    
    public override init(frame: CGRect) {
        super.init(frame: frame)
        setupUI()
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    deinit {
        removeKeyboardObserver()
    }
    
    func clearInput() {
        textView.text = ""
    }
    
    func dismissKeyboard() {
        textView.resignFirstResponder()
    }
    
    func markReceiveAction() {
        receiveActionFlag = true
    }
    
    func addKeyboardObserver() {
        if didAddObserver { return }
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
        didAddObserver = true
    }
    
    func removeKeyboardObserver() {
        NotificationCenter.default.removeObserver(self)
        didAddObserver = false
    }
    
    // MARK: - private
    private let topLineView = {
        let view = UIView()
        view.backgroundColor = UIColor.bjl_color(withHexString: "#EEEEEE")
        return view
    }()
    
    private let textView = {
        let textView = UITextView()
        textView.font = .systemFont(ofSize: 14)
        textView.bjl_placeholderLabel.text = BJLLocalizedString("参与讨论")
        textView.bjl_placeholderColor = UIColor.bjl_color(withHexString: "#999999")
        textView.textColor = UIColor.bjl_color(withHexString: "#333333")
        textView.backgroundColor = UIColor.bjl_color(withHexString: "#F2F3F9")
        textView.layer.cornerRadius = 2
        textView.layer.masksToBounds = true
        return textView
    }()
    
    private let sendButton = {
        let button = UIButton(type: .custom)
        button.setTitle(BJLLocalizedString("发布"), for: .normal)
        button.contentVerticalAlignment = .center
        button.setTitleColor(UIColor.bjl_color(withHexString: "#2b53dd"), for: .normal)
        button.backgroundColor = .clear
        return button
    }()
    
    private var didAddObserver = false,
                receiveActionFlag = true
    
    @objc func keyboardWillShow(_ notification: Notification) {
        guard let userInfo = notification.userInfo else { return }
        
        self.updateTextViewHeightAsInputMode(true)
        let keyboardHeight = (userInfo[UIResponder.keyboardFrameEndUserInfoKey] as? CGRect)?.height ?? 0
        updateYOffset(-Float(keyboardHeight), animation: true)
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        self.updateTextViewHeightAsInputMode(false)
        updateYOffset(0, animation: true)
    }

    func updateYOffset(_ yOffset: Float, animation: Bool) {
        textView.snp.updateConstraints { make in
            make.bottom.equalToSuperview().offset(yOffset)
        }
        if animation {
            UIView.animate(withDuration: 0.25) {
                self.superview?.layoutIfNeeded()
            }
        }
    }
}

fileprivate extension CommentInputView {
    func setupUI() {
        backgroundColor = .white
        
        topLineView.layout(in: self) { make in
            make.left.top.right.equalToSuperview()
            make.height.equalTo(1)
        }
        addSubview(sendButton)
        textView.layout(in: self) { make in
            make.top.left.equalToSuperview().offset(15)
            make.bottom.equalToSuperview().offset(-15)
            make.right.equalTo(sendButton.snp.left)
            make.bottom.equalToSuperview().offset(-15)
            make.height.equalTo(35)
        }
        sendButton.layout { make in
            make.right.equalToSuperview()
            make.centerY.equalTo(textView)
            make.width.equalTo(65)
        }
        sendButton.bjl_addHandler { [weak self] button in
            guard let self = self, self.receiveActionFlag else { return }
            guard self.textView.text.count > 0 else {
                self.textView.resignFirstResponder()
                return
            }
            self.sendActionCallback?(self.textView.text)
            self.receiveActionFlag = false
        }
    }
    
    func updateTextViewHeightAsInputMode(_ inputMode: Bool) {
        textView.snp.updateConstraints { make in
            make.height.equalTo(inputMode ? 70 : 35)
        }
    }
}
