//
//  CommentListViewController.swift
//  BJVideoPlayerUI
//
//  Created by ney on 2023/6/5.
//  Copyright © 2023 BaijiaYun. All rights reserved.
//

import UIKit
import MJRefresh
import BJLiveBase
import BJVideoPlayerCore

public class CommentListViewController: UIViewController {
    // MARK: - api
    @objc public init(videoID: String) {
        super.init(nibName: nil, bundle: nil)
        self.videoID = videoID
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    public override func viewDidLoad() {
        super.viewDidLoad()
        buildUI()
        loadFirstPageData(showHUD: true)
    }
    
    deinit {
        commentInputView.removeKeyboardObserver()
    }
    
    // MARK: - private
    private let pageSize = 20
    private var page = 1,
                dataSource = [BJVCommentItem](),
                commentInputView = CommentInputView(),
                videoID = ""
    
    private let tableView: UITableView = {
        let view = UITableView(frame: .zero, style: .plain)
        view.separatorStyle = .none
        view.backgroundColor = .white
        view.register(CommentListCell.self, forCellReuseIdentifier: "CommentListCell")
        view.estimatedRowHeight = UITableView.automaticDimension
        if #available(iOS 11.0, *) {
            view.insetsContentViewsToSafeArea = false
        }
        return view
    }()
    let emptyView: EmptyView = {
        let view = EmptyView()
        view.isHidden = true
        return view
    }()
    
    func buildUI() {
        tableView.delegate = self
        tableView.dataSource = self
        tableView.layout(in: view) { make in
            make.edges.equalToSuperview()
        }
        
        emptyView.layout(in: view) { make in
            make.center.equalTo(tableView)
        }
        
        let header = MJRefreshNormalHeader { [weak self] in
            self?.loadFirstPageData()
        }
        header.stateLabel?.isHidden = true
        header.lastUpdatedTimeLabel?.isHidden = true
        tableView.mj_header = header
        
        commentInputView.layout(in: view) { make in
            make.left.right.bottom.equalToSuperview()
        }
        commentInputView.addKeyboardObserver()
        commentInputView.sendActionCallback = { [weak self] text in
            guard let self = self else { return }
            
            BJVCommentManager.sharedInstance().leaveComment(withVideoID: self.videoID, comment: text, avatar: "") { error in
                if let error = error {
                    self.showHUD(with: error.localizedDescription)
                }
                else {
                    self.commentInputView.clearInput()
                    self.commentInputView.dismissKeyboard()
                    self.showHUD(with: BJLLocalizedString("评论成功"))
                    self.loadFirstPageData()
                }
                self.commentInputView.markReceiveAction()
            }
        }
    }
}

// MARK: - table view
extension CommentListViewController: UITableViewDataSource, UITableViewDelegate {
    public func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataSource.count
    }
    
    public func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "CommentListCell", for: indexPath)
        guard let commentCell = cell as? CommentListCell else { return UITableViewCell() }
        let data = dataSource[indexPath.row]
        commentCell.update(model: data)
        
        commentCell.deleteActionHandler = { [weak self] cell, data in
            guard let self = self, let commentID = data?.commentID else { return }
            
            self.showDeleteConfirmAlert {
                BJVCommentManager.sharedInstance().deleteComment(withVideoID: self.videoID, commentID: commentID) { error in
                    if error == nil, let index = self.tableView.indexPath(for: cell) {
                        self.dataSource.remove(at: index.row)
                        self.tableView.deleteRows(at: [index], with: .automatic)
                        self.checkListIsEmpty()
                    }
                    else if let error = error {
                        self.showHUD(with: error.localizedDescription)
                    }
                }
            }
        }
        
        commentCell.likeActionHandler = { [weak self] cell, data, like in
            guard let self = self, let data = data else { return }
            BJVCommentManager.sharedInstance().likeComment(withVideoID: self.videoID, commentID: data.commentID ?? "", like: like) { error in
                if error == nil,
                   let index = self.tableView.indexPath(for: cell),
                   index.row < self.dataSource.count {
                    let item = self.dataSource[index.row]
                    item.like = like
                    self.tableView.reloadRows(at: [indexPath], with: .automatic)
                }
                else if let error = error {
                    self.showHUD(with: error.localizedDescription)
                }
            }
        }
        
        return commentCell
    }
    
    public func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
    }
}

extension CommentListViewController {
    func syncData(_ data: [BJVCommentItem], append: Bool) {
        if !append { dataSource.removeAll() }
        dataSource.append(contentsOf: data)
        refreshTableView()
    }
    
    func refreshTableView() {
        tableView.reloadData()
        checkListIsEmpty()
    }
    
    func checkListIsEmpty() {
        emptyView.isHidden = (dataSource.count != 0)
    }
    
    func loadFirstPageData(showHUD: Bool = false) {
        page = 1
        loadData(page: page, showHUD: showHUD) { [weak self] modeList in
            guard let self = self else { return }
            
            self.syncData(modeList, append: false)
            
            if modeList.count == self.pageSize {
                self.page += 1
                
                let footer = MJRefreshAutoNormalFooter {
                    self.loadNextPageData()
                }
                footer.setTitle(BJLLocalizedString("没有更多了"), for: .noMoreData)
                self.tableView.mj_footer = footer
            }
            else {
                self.tableView.mj_footer = nil
            }
            
            self.tableView.mj_header?.endRefreshing()
            self.tableView.mj_totalDataCount()
            
            if self.commentInputView.frame.size.height > 0 {
                self.tableView.contentInset = .init(top: 0, left: 0, bottom: self.commentInputView.frame.size.height, right: 0)
            }
        }
    }
    
    func loadNextPageData() {
        loadData(page: page, showHUD: false) { [weak self] modeList in
            guard let self = self else { return }
            
            self.syncData(modeList, append: true)
            
            if modeList.count == self.pageSize {
                self.page += 1
                self.tableView.mj_footer?.endRefreshing()
            }
            else {
                self.tableView.mj_footer?.endRefreshingWithNoMoreData()
            }
        }
    }
    
    func loadData(page: Int = 1, showHUD: Bool, result: (([BJVCommentItem]) -> Void)?) {
        BJLProgressHUD.hide(for: self.view, animated: true)
        
        if showHUD {
            BJLProgressHUD.bjl_showForLoading(withSuperview: view, animated: true)
        }
        
        BJVCommentManager.sharedInstance().requestComment(withVideoID: self.videoID, page: page, pageSize: pageSize) { [weak self] data, error in
            guard let self = self, let data = data else {
                if let view = self?.view {
                    BJLProgressHUD.hide(for: view, animated: true)
                }
                result?([])
                return
            }
            
            BJLProgressHUD.hide(for: self.view, animated: true)
            
            result?(data)
        }
    }
    
    func showDeleteConfirmAlert(_ callback: (() -> Void)?) {
        let alert = UIAlertController(title: nil, message: BJLLocalizedString("确定删除该评论？"), preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: BJLLocalizedString("取消"), style: .cancel) { _ in
        })
        alert.addAction(UIAlertAction(title: BJLLocalizedString("确定"), style: .default ) { _ in
            callback?()
        })
        present(alert, animated: true, completion: nil)
    }
    
    func showHUD(with text: String?) {
        guard let view = UIApplication.shared.windows.last(where: { $0.isKeyWindow }) ?? view else { return }
        BJLProgressHUD.bjl_show(forText: text ?? "", superview: view, animated: true)
    }
    
}

extension CommentListViewController {
    class EmptyView: UIView {
        override init(frame: CGRect) {
            super.init(frame: frame)
            
            imageView.layout(in: self) { make in
                make.top.left.right.equalToSuperview().offset(0)
            }
            titleLabel.layout(in: self) { make in
                make.top.equalTo(imageView.snp.bottom).offset(10)
                make.centerX.equalToSuperview()
                make.bottom.equalToSuperview()
            }
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        private lazy var imageView: UIImageView = {
            let imageView = UIImageView(image: UIImage.bjpu_imageNamed("comment_list_empty"))
            imageView.contentMode = .scaleAspectFit
            return imageView
        }()
        
        private lazy var titleLabel: UILabel = {
            let label = UILabel()
            label.font = UIFont.systemFont(ofSize: 13)
            label.textColor = .bjl_color(hex: 0x333333)
            label.textAlignment = .center
            label.text = BJLLocalizedString("暂无评论")
            return label
        }()
    }
}
