//
//  TripartiteLayoutOutlinesViewController.swift
//  BJVideoPlayerUI
//
//  Created by ney on 2022/12/23.
//  Copyright © 2022 BaijiaYun. All rights reserved.
//

import UIKit
import BJLiveBase
import BJSwiftBase
import SnapKit
import ExtSwift
import ExCodable
import BJVideoPlayerCore

public class TripartiteLayoutOutlinesViewController: UIViewController {
    static func generatePPTURL(ppt: BJVTripartiteLayoutPPT, forPage page: Int) -> String {
        let url = "\(ppt.baseURL)/\(ppt.fid)_\(ppt.sn)_\(page).png"
        return url
    }

    // MARK: - open
    @objc public required init(playerManager: BJVPlayerManager, outlines: BJVTripartiteLayoutInfo) {
        self.playerManager = playerManager
        
        if let outlinesData = outlines.copy() as? BJVTripartiteLayoutInfo {
            var sortedTxtByTime = outlinesData.txt.content
            sortedTxtByTime.sort { a, b in
                return a.time < b.time
            }
            outlinesData.txt.updateContent(sortedTxtByTime)
            self.outlines = outlinesData
        }
        else {
            self.outlines = outlines
        }
        
        super.init(nibName: nil, bundle: nil)
        addObserverForPlayer(self.playerManager)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    public override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    @objc
    public var swapViewEventHandler: ((UIView, Bool) -> UIView?)?
    
    // MARK: - private
    private let playerManager: BJVPlayerManager
    private let outlines: BJVTripartiteLayoutInfo
    
    // top
    private let imageContainer: UIView = UIView { view in
    }
    private let swappableImageView: UIImageView = UIImageView { view in
        view.contentMode = .scaleAspectFit
        view.backgroundColor = .clear
        view.clipsToBounds = true
    }
    private let swapContainerContentButton: UIButton = UIButton { btn in
        btn.bjl_setImage(UIImage.bjpu_imageNamed("tripartite_layout_cell_swap"), for: .normal)
    }
    
    // mid
    private let tabView: TabView = TabView { tab in
    }
    
    // bottom
    private let tableView: UITableView = {
        let view = UITableView(frame: .zero, style: .plain)
        view.separatorStyle = .none
        view.backgroundColor = .white
        view.register(OutlineCell.self, forCellReuseIdentifier: "OutlineCell")
        if #available(iOS 11.0, *) {
            view.insetsContentViewsToSafeArea = false
        }
        return view
    }()
    
    // other
    private var currentHighlightedIndex = -1
    private var lastSelectionTime: TimeInterval = 0.0
    
    func setupUI() {
        self.view.clipsToBounds = true
        
        // top
        imageContainer.layout(in: view) { make in
            make.left.top.right.equalToSuperview()
            make.height.equalTo(view.snp.width).multipliedBy(9.0 / 16.0)
        }
        swapContainerContentButton.layout(in: view) { make in
            make.left.equalTo(imageContainer)
            make.bottom.equalTo(imageContainer)
        }
        swappableImageView.layout(in: imageContainer) { make in
            make.edges.equalToSuperview().priority(.high)
        }
        
        // mid
        tabView.layout(in: view) { make in
            make.top.equalTo(imageContainer.snp.bottom)
            make.left.right.equalToSuperview()
            make.height.equalTo(27)
        }
        
        // bottom
        tableView.layout(in: view) { make in
            make.top.equalTo(tabView.snp.bottom)
            make.left.bottom.right.equalToSuperview()
        }
        
        swapContainerContentButton.bjl_addHandler { [unowned self] _ in
            let didSwap = self.swappableImageView.superview != self.imageContainer
            let view = self.swapViewEventHandler?(self.swappableImageView, didSwap)
            if !didSwap, let view = view {
                view.layout(in: self.imageContainer) { make in
                    make.edges.equalToSuperview()
                }
            }
            else if didSwap {
                let subviews = self.imageContainer.subviews
                for i in subviews {
                    i.removeFromSuperview()
                }
                swappableImageView.layout(in: self.imageContainer) { make in
                    make.edges.equalToSuperview()
                }
            }
        }
        tabView.bjl_addHandler(for: .valueChanged) { [unowned self] tab, event in
            tableView.reloadData()
        }
        tableView.delegate = self
        tableView.dataSource = self
        changeHighlightingPage(currentHighlightedIndex, animation: false, seek: false)
    }
    
    func addObserverForPlayer(_ player: BJVPlayerManager) {
        bjl_kvo(BJLMakeProperty(player, #selector(getter: player.currentTime)), observer: KVO(type: TimeInterval.self) { [weak self] _, _, _ in
            guard let self = self else { return .stop }
            
            let now = NSDate().timeIntervalSince1970
            // seek 时，存在关键帧导致的时间不精确，会回到选中的 ppt 的以前的时间，导致 ppt 来回跳页
            // 加 2s 间隔，保证在点击翻页以后，不刷新 ppt 选中状态
            if now - self.lastSelectionTime < 2 { return .keep }
            
            let index = self.findCorrespondingPPTIndexByTimeOffset(self.playerManager.currentTime)
            if index != self.currentHighlightedIndex {
                self.changeHighlightingPage(index, animation: true, seek: false)
            }
            
            return .keep
        })
    }
    
    func findCorrespondingPPTIndexByTimeOffset(_ timeOffset: TimeInterval) -> Int {
        var page = 0
        for i in self.outlines.txt.content.enumerated() {
            if timeOffset < TimeInterval(i.element.time) { break }
            page = i.offset
        }
        return page
     }
    
    func changeHighlightingPage(_ page: Int, animation: Bool, seek: Bool) {
        guard outlines.txt.content.count > 0, page >= 0 else { return }
        
        let data = outlines.txt.content[page]
        let url = Self.generatePPTURL(ppt: self.outlines.ppt, forPage: data.page)
        self.swappableImageView.bjl_setImage(with: URL(string: url))
        currentHighlightedIndex = page
        tableView.reloadData()
        
        if animation {
            scrollTableViewRowToCenter(row: page)
        }
        
        if seek {
            self.playerManager.seek(TimeInterval(data.time))
        }
    }
    
    func scrollTableViewRowToCenter(row: Int) {
        guard row >= 0 else { return }
        let validTableViewSize = tableView.frame.size.width > 0.0 && tableView.frame.size.height > 0.0
        if validTableViewSize {
            tableView.scrollToRow(at: IndexPath(row: row, section: 0), at: .middle, animated: true)
        }
    }
}

// MARK: - table view
extension TripartiteLayoutOutlinesViewController: UITableViewDelegate, UITableViewDataSource {
    public func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return outlines.txt.content.count
    }
    
    public func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OutlineCell", for: indexPath)
        guard let outlineCell = cell as? OutlineCell else { return UITableViewCell() }
        let data = outlines.txt.content[indexPath.row]
        outlineCell.update(ppt: outlines.ppt, outlineItem: data, index:indexPath.row, showPreview: tabView.selectedType == .preview)
        outlineCell.updateHighlightState(currentHighlightedIndex == indexPath.row)
        outlineCell.selectionStyle = .none
        return outlineCell
    }
    
    public func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        lastSelectionTime = Date().timeIntervalSince1970
        changeHighlightingPage(indexPath.row, animation: false, seek: true)
    }
}

// MARK: - widget
extension TripartiteLayoutOutlinesViewController {
    class TabView: UIControl {
        
        enum TabType {
            case outline
            case preview
        }
        
        required init() {
            super.init(frame: .zero)
            setupUI()
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        // ui var
        private let outlineButton: UIButton = UIButton { btn in
            btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .medium)
            btn.contentHorizontalAlignment = .center
            btn.backgroundColor = .white
            btn.setTitleColor(.bjl_color(withHexString: "#4A4A4A"), for: .normal)
            btn.setTitleColor(.bjl_color(withHexString: "#1E5EFF"), for: .selected)
            btn.setTitle(BJLLocalizedString("纲要"), for: .normal)
            btn.isSelected = true
        }
        private let outlineSelectionView: UIView = UIView { view in
            view.isUserInteractionEnabled = false
            view.backgroundColor = .bjl_color(withHexString: "#1E5EFF")
        }
        
        private let previewButton: UIButton = UIButton { btn in
            btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .medium)
            btn.contentHorizontalAlignment = .center
            btn.backgroundColor = .white
            btn.setTitleColor(.bjl_color(withHexString: "#4A4A4A"), for: .normal)
            btn.setTitleColor(.bjl_color(withHexString: "#1E5EFF"), for: .selected)
            btn.setTitle(BJLLocalizedString("预览"), for: .normal)
        }
        private let previewSelectionView: UIView = UIView { view in
            view.isUserInteractionEnabled = false
            view.backgroundColor = .clear
        }
        
        private let bottomLineView: UIView = UIView { view in
            view.backgroundColor = .bjl_color(withHexString: "#DCDEE2")
        }
        
        var selectedType = TabType.outline
        
        // layout
        func setupUI() {
            outlineButton.layout(in: self) { make in
                make.top.left.bottom.equalToSuperview()
                make.width.equalToSuperview().dividedBy(2)
            }
            previewButton.layout(in: self) { make in
                make.top.right.bottom.equalToSuperview()
                make.width.equalToSuperview().dividedBy(2)
            }
            bottomLineView.layout(in: self) { make in
                make.left.bottom.right.equalToSuperview()
                make.height.equalTo(1)
            }
            outlineSelectionView.layout(in: self) { make in
                make.left.bottom.right.equalTo(outlineButton)
                make.height.equalTo(2)
            }
            previewSelectionView.layout(in: self) { make in
                make.left.bottom.right.equalTo(previewButton)
                make.height.equalTo(2)
            }
            
            outlineButton.bjl_addHandler { [unowned self] btn in
                btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .medium)
                btn.isSelected = true
                previewButton.isSelected = false
                sendActions(for: .valueChanged)
                previewButton.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .regular)
                outlineSelectionView.backgroundColor = .bjl_color(withHexString: "#1E5EFF")
                previewSelectionView.backgroundColor = .clear
                selectedType = .outline
            }
            
            previewButton.bjl_addHandler { [unowned self] btn in
                btn.isSelected = true
                outlineButton.isSelected = false
                btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .medium)
                sendActions(for: .valueChanged)
                outlineButton.titleLabel?.font = UIFont.systemFont(ofSize: 12.0, weight: .regular)
                previewSelectionView.backgroundColor = .bjl_color(withHexString: "#1E5EFF")
                outlineSelectionView.backgroundColor = .clear
                selectedType = .preview
            }
        }
    }
    
    class OutlineCell: UITableViewCell {
        private let backgroundColorView = UIView { view in
            view.clipsToBounds = true
            view.layer.cornerRadius = 4
        }
        private let titleLabel = UILabel { lb in
            lb.backgroundColor = .clear
            lb.font = .systemFont(ofSize: 12, weight: .regular)
            lb.numberOfLines = 2
        }
        private let stateImageView = UIImageView { icon in
            icon.setContentCompressionResistancePriority(.required, for: .horizontal)
        }
        private let previewImageView = UIImageView { img in
            img.layer.masksToBounds = true
            img.layer.cornerRadius = 4
            img.contentMode = .scaleAspectFill
        }
        
        override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
            super.init(style: style, reuseIdentifier: reuseIdentifier)
            
            backgroundColorView.layout(in: self.contentView) { make in
                make.top.equalToSuperview().offset(1)
                make.left.equalToSuperview().offset(4)
                make.bottom.equalToSuperview().offset(-1)
                make.right.equalToSuperview().offset(-4)
            }
            contentView.addSubview(stateImageView)
            titleLabel.layout(in: self.contentView) { make in
                make.top.equalToSuperview().offset(6)
                make.left.equalToSuperview().offset(8)
                make.right.lessThanOrEqualTo(stateImageView.snp.left).offset(-8)
                make.bottom.equalToSuperview().offset(-6)
            }
            stateImageView.layout { make in
                make.centerY.equalTo(titleLabel)
                make.right.equalToSuperview().offset(-8)
            }
            previewImageView.layout(in: self.contentView) { make in
                make.top.equalTo(titleLabel.snp.bottom).offset(3)
                make.left.equalTo(titleLabel)
                make.right.equalTo(stateImageView)
                if UI_USER_INTERFACE_IDIOM() == .pad {
                    make.height.equalTo(100)
                }
                else {
                    make.height.equalTo(74)
                }
            }
        }
        
        private var isPreview = false
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        func update(ppt: BJVTripartiteLayoutPPT, outlineItem: BJVTripartiteLayoutTxtItem, index: Int, showPreview: Bool) {
            titleLabel.text = !(outlineItem.outline.isEmpty) ? outlineItem.outline : "第\(index + 1)页"
            previewImageView.isHidden = !showPreview
            if showPreview {
                let url = generatePPTURL(ppt: ppt, forPage: outlineItem.page)
                previewImageView.bjl_setImage(with: URL(string: url))
            }
            if isPreview != showPreview {
                titleLabel.snp.updateConstraints { make in
                    let imageHeight = UI_USER_INTERFACE_IDIOM() == .pad ? 100 : 74
                    make.bottom.equalToSuperview().offset(showPreview ? -(imageHeight + 8) : -6)
                }
                isPreview = showPreview
            }
        }
        
        func updateHighlightState(_ isHighlighted: Bool) {
            self.backgroundColorView.backgroundColor = isHighlighted ? .bjl_color(withHexString: "#1E5EFF") : .bjl_color(withHexString: "#C7D0E6")?.withAlphaComponent(0.1)
            self.titleLabel.textColor = .bjl_color(withHexString: isHighlighted ? "#FFFFFF" : "#999999")
            let iconName = isHighlighted ? "tripartite_layout_cell_play_icon_white" : "tripartite_layout_cell_play_icon_gray"
            self.stateImageView.image = UIImage.bjpu_imageNamed(iconName)
        }
    }
}
